#ifndef __ITFMAP_H__
#define __ITFMAP_H__

#include <windows.h>

/**************************************************************************
NAME         : INTERFACE_ENTRY
TYP          : Strukturtyp
AUFGABE      : tabellarische Strukturierung von Interfacezeigern und -offsets
**************************************************************************/
typedef struct tagINTERFACE_ENTRY
{
	const IID *pIID;          
	const ULONG_PTR dwOffset; //Offset eines Interfaces innerhalb einer Klasse
} INTERFACE_ENTRY;

/**************************************************************************
NAME         : BASE_OFFSET
TYP          : Hilfsmakro 
AUFGABE      : Berechnung des Offsets eines Interfaces innerhalb einer Klasse
IN           : Itf       ... Name des Interface
               ClassName ... Name der Implementationsklasse
**************************************************************************/
#define BASE_OFFSET(Itf, ClassName) (((ULONG_PTR)(static_cast<ClassName*>((Itf*)0x10001000)))-0x10001000)

/**************************************************************************
NAME         : BEGIN_INTERFACE_MAP
TYP          : Makro (Tabellenkopf)
AUFGABE      : Beginnt die Tabelle der von einer COM-Klasse untersttzten Interfaces
IN           : ClassName ... Name der COM-Klasse
**************************************************************************/
#define BEGIN_INTERFACE_MAP(ClassName) \
	typedef ClassName _ITCls; \
	const INTERFACE_ENTRY *GetInterfaceTable() \
	{ \
		static const INTERFACE_ENTRY table[]={ \

/**************************************************************************
NAME         : IMPLEMENTS_INTERFACE
TYP          : Makro (Tabelleneintrag)
AUFGABE      : Definiert ein von einer COM-Klasse untersttztes Interface
IN           : Itf ... Name des einzutragenden Interface
**************************************************************************/
#define IMPLEMENTS_INTERFACE(Itf) \
			{ &IID_##Itf, BASE_OFFSET(_ITCls,Itf) }, \

/**************************************************************************
NAME         : IMPLEMENTS_INTERFACE_AS
TYP          : Makro (Tabelleneintrag)
AUFGABE      : Definiert ein von einer COM-Klasse untersttztes Interface (mit Aliasnamen)
IN           : Itf   ... Klassename des einzutragenden Interface
               alias ... COM-Aliasname des einzutragenden Interface
**************************************************************************/
#define IMPLEMENTS_INTERFACE_AS(alias, Itf) \
			{ &IID_##alias, BASE_OFFSET(_ITCls,Itf) }, \

/**************************************************************************
NAME         : END_INTERFACE_MAP
TYP          : Makro (Tabellenende)
AUFGABE      : Beendet die Tabelle der von einer COM-Klasse untersttzten Interfaces
**************************************************************************/
#define END_INTERFACE_MAP() \
			{ 0, -1 } }; \
		return table; \
	} \

/**************************************************************************
NAME         : IMPLEMENT_UNKNOWN
TYP          : Makro
AUFGABE      : Implementiert die Schnittstelle IUnknown (fr alle COM-Klassen)
**************************************************************************/
#define IMPLEMENT_UNKNOWN() \
	typedef struct tagLONG0 { \
		LONG Value; \
		tagLONG0() { Value=0; } \
	} LONG0; \
	LONG0 m_cRef; \
	\
	virtual STDMETHODIMP QueryInterface(REFIID riid, LPVOID *ppv) \
	{ \
		if (!ppv) return E_POINTER; \
		const INTERFACE_ENTRY *pTable=GetInterfaceTable(); \
		if (IsEqualGUID(riid,IID_IUnknown)) \
		{ \
			*ppv=((char*)this)+pTable->dwOffset; \
			((IUnknown*)*ppv)->AddRef(); \
			return S_OK; \
		} \
		while (pTable->dwOffset!=-1) \
		{ \
			if (IsEqualGUID(riid,*pTable->pIID)) \
			{ \
				*ppv=((char*)this)+pTable->dwOffset; \
				((IUnknown*)*ppv)->AddRef(); \
				return S_OK; \
			} \
			pTable++; \
		} \
		*ppv=NULL; \
		return E_NOINTERFACE; \
	} \
	\
	virtual STDMETHODIMP_(ULONG) AddRef() \
	{ return InterlockedIncrement(&m_cRef.Value); } \

/**************************************************************************
NAME         : RELEASE_DYNAMIC
TYP          : Makro
AUFGABE      : Implementiert die Methode Release (fr dynamische COM-Klassen)
**************************************************************************/
#define RELEASE_DYNAMIC() \
	virtual STDMETHODIMP_(ULONG) Release() \
	{ \
		ULONG cRef=InterlockedDecrement(&m_cRef.Value); \
		if (cRef==0) delete this; \
		return cRef; \
	} \

/**************************************************************************
NAME         : RELEASE_STATIC
TYP          : Makro
AUFGABE      : Implementiert die Methode Release (fr statische COM-Klassen)
**************************************************************************/
#define RELEASE_STATIC() \
	virtual STDMETHODIMP_(ULONG) Release() \
	{ return (m_cRef.Value>1)?InterlockedDecrement(&m_cRef.Value):1; } \

#endif