#ifndef __REGTABLE_H__
#define __REGTABLE_H__

#include <windows.h>
#include <olectl.h>

/**************************************************************************
NAME         : REGISTRY_ENTRY
TYP          : Strukturtyp
AUFGABE      : tabellarische Strukturierung von Systemregistrierungseintrgen
**************************************************************************/
typedef struct tagREGISTRY_ENTRY
{
	const char *key;
	const char *value;
	const char *data;
} REGISTRY_ENTRY;

/**************************************************************************
NAME         : BEGIN_REGISTRY_TABLE
TYP          : Makro (Tabellenkopf)
AUFGABE      : Beginnt die Tabelle der COM-Systemregistrierungseintrge
**************************************************************************/
#define BEGIN_REGISTRY_TABLE() \
	const REGISTRY_ENTRY *GetRegTable() \
	{ \
		static const REGISTRY_ENTRY regTable[]={ \
			{ 0, 0, 0 },

/**************************************************************************
NAME         : REGISTER_CLASSID
TYP          : Makro (Tabelleneintrag)
AUFGABE      : Definiert einen Registry-Eintrag fr eine COM-Klasse
IN           : ClassID     ... CLSID der zu registrierenden COM-Klasse
               Description ... Kurzbeschreibung der zu registrierenden COM-Klasse
**************************************************************************/
#define REGISTER_CLASSID(ClassID, Description) \
			{ "CLSID\\{"##ClassID"}", 0, Description }, \

/**************************************************************************
NAME         : REGISTER_CLASS_INPROC
TYP          : Makro (Tabelleneintrag)
AUFGABE      : Definiert einen Registry-Eintrag fr den Pfad zu COM-Klasse (in einer DLL)
IN           : ClassID  ... CLSID der zu registrierenden COM-Klasse
               FilePath ... Pfad zur zu registrierenden COM-Klasse
**************************************************************************/
#define REGISTER_CLASS_INPROC(ClassID, FilePath) \
			{ "CLSID\\{"##ClassID"}\\InprocServer32", 0, FilePath }, \

/**************************************************************************
NAME         : REGISTER_CLASS_LOCAL
TYP          : Makro (Tabelleneintrag)
AUFGABE      : Definiert einen Registry-Eintrag fr den Pfad zu COM-Klasse (in einem Executable)
IN           : ClassID  ... CLSID der zu registrierenden COM-Klasse
               FilePath ... Pfad zur zu registrierenden COM-Klasse
**************************************************************************/
#define REGISTER_CLASS_LOCAL(ClassID, FilePath) \
			{ "CLSID\\{"##ClassID"}\\LocalServer32", 0, FilePath }, \

/**************************************************************************
NAME         : REGISTER_CLASS_PROGID
TYP          : Makro (Tabelleneintrag)
AUFGABE      : Definiert einen Registry-Eintrag fr die PROGID einer COM-Klasse
IN           : ClassID     ... CLSID der zu registrierenden COM-Klasse
               ProgID      ... ProgID der zu registrierenden COM-Klasse
               Description ... Kurzbeschreibung der zu registrierenden COM-Klasse
**************************************************************************/
#define REGISTER_CLASS_PROGID(ClassID, ProgID, Description) \
			{ "CLSID\\{"##ClassID"}\\ProgID", 0, ProgID }, \
			{ ProgID, 0, Description }, \
			{ ##ProgID"\\CLSID", 0, "{"##ClassID"}" }, \

/**************************************************************************
NAME         : REGISTER_CLASS_VIPROGID
TYP          : Makro (Tabelleneintrag)
AUFGABE      : Definiert einen Registry-Eintrag fr die versionsunabhngige PROGID einer COM-Klasse
IN           : ClassID     ... CLSID der zu registrierenden COM-Klasse
               VIProgID    ... versionsunabhngige ProgID der zu registrierenden COM-Klasse
							 Version     ... aktuelle Version der COM-Klasse
               Description ... Kurzbeschreibung der zu registrierenden COM-Klasse
**************************************************************************/
#define REGISTER_CLASS_VIPROGID(ClassID, VIProgID, Version, Description) \
			{ "CLSID\\{"##ClassID"}\\VersionIndependentProgID", 0, VIProgID }, \
			{ VIProgID, 0, Description }, \
			{ ##VIProgID"\\CLSID", 0, "{"##ClassID"}" }, \
			{ ##VIProgID"\\CurVer", 0, ##VIProgID"."#Version }, \

/**************************************************************************
NAME         : END_REGISTRY_TABLE
TYP          : Makro (Tabellenende)
AUFGABE      : Beendet die Tabelle der COM-Systemregistrierungseintrge
**************************************************************************/
#define END_REGISTRY_TABLE() \
			{ 0, 0, 0 } }; \
		return (regTable+1); \
	} \

/**************************************************************************
NAME         : IMPLEMENT_REGISTERSERVER
TYP          : Makro
AUFGABE      : Implementiert DllRegisterServer
**************************************************************************/
#define IMPLEMENT_REGISTERSERVER() \
	STDAPI DllUnregisterServer(); \
	\
	STDAPI DllRegisterServer() \
	{ \
		const REGISTRY_ENTRY *rTable=GetRegTable(); \
		while (rTable->key) \
		{ \
			HKEY hkey; \
			LONG err=RegCreateKeyA(HKEY_CLASSES_ROOT,rTable->key,&hkey); \
			if (err==ERROR_SUCCESS) \
			{ \
				err=RegSetValueExA(hkey,rTable->value,0,REG_SZ,(CONST BYTE*)rTable->data,(DWORD)strlen(rTable->data)+1); \
				RegCloseKey(hkey); \
			} else { \
				DllUnregisterServer(); \
				return SELFREG_E_CLASS; \
			} \
			rTable++; \
		} \
		return S_OK; \
	} \

/**************************************************************************
NAME         : IMPLEMENT_UNREGISTERSERVER
TYP          : Makro
AUFGABE      : Implementiert DllUnregisterServer
**************************************************************************/
#define IMPLEMENT_UNREGISTERSERVER() \
	STDAPI DllUnregisterServer() \
	{ \
		HRESULT hr=S_OK; \
		const REGISTRY_ENTRY *rTable=GetRegTable(); \
		while (rTable->key) rTable++; \
		while ((--rTable)->key) \
		{ \
			LONG err=RegDeleteKeyA(HKEY_CLASSES_ROOT,rTable->key); \
			if (err!=ERROR_SUCCESS) hr=S_FALSE; \
		} \
		return hr; \
	} \

#endif